# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["VideoModel"]

VideoModel: TypeAlias = Literal["sora-2", "sora-2-pro"]
